/**
@file
@date        17/11/2014
@author:     Josef Hofer
@copyright   CERN (EN-ICE-SCD)

@brief Library containing functions to retrieve configuration data for unLHCLoggingDB.

The functions are receiving the full data point element name as an input parameter
and are returning name, hierarchy, alias and element extension as output parameters

Usage: Public

PVSS manager usage: CTRL, UI

Constraints:
- PVSS version: 3.11
- Operating System: Linux
*/


/// set sHierarchy to 'ERROR', if information cannot be retrieved for the given DPE
public const string cs_unLHCLoggingDBUserLib_Error  = "ERROR";

/// set sHierarchy to 'IGNORE', if you want to ignore a certain name (eg. for SPARE or RESERVE)
public const string cs_unLHCLoggingDBUserLib_Ignore = "IGNORE";

/// string constant to build hierarchy for vacuum application {sblancha} 2017-12-19
private const string cs_unLHCLoggingDBUserLib_Activity = "VAC";

/**
This is the very basic function for retrieving logging config data. 
It can be used for developement purpose, ore if there is no user function
and the devices do not fit into the UNICOS naming scheme.

@param sFullDpeName     input, full data point element name

@param sName            output, variable name
@param sDescription     output, description
@param sHierarchy       output, systemName/dpType
@param sDpAlias         output, alias
@param sDpeExtension    output, element extension

*/
public void unLHCLoggingDBUserLib_getDeviceInfo_BASIC(
    string sFullDpeName,
    string &sName,
    string &sDescription,
    string &sHierarchy,
    string &sDpAlias,
    string &sDpeExtension)
{
    string sSystemName = getSystemName();

    sName = sFullDpeName; // TOCHECK can this ever be UNIQUE in META_VARIABLES
    sHierarchy = substr(sSystemName, 0, strpos(sSystemName, ":")) + "/" + dpTypeName(sFullDpeName);
    sDescription = dpGetDescription(sFullDpeName, 0);
    sDpAlias = "";
    sDpeExtension = sFullDpeName;
}

/**
This is the default function for retrieving logging config data for UNICOS devices.

sDpAlias + sDpeExtension is used to display the sFullDpeName in a more human readable form.

@param sFullDpeName     input, full data point element name

@param sName            output, variable name
@param sDescription     output, description
@param sHierarchy       output, Activity/Location/Application/Nature/Domain;
@param sDpAlias         output, alias
@param sDpeExtension    output, element extension

*/
public void unLHCLoggingDBUserLib_getDeviceInfo_UNICOS(
    string sFullDpeName,
    string &sName,
    string &sDescription,
    string &sHierarchy,
    string &sDpAlias,
    string &sDpeExtension)
{
    dyn_string dsParameters, dsSplit;
    string sLocation, sApplication, sDomaine, sNature;
    
    sHierarchy = cs_unLHCLoggingDBUserLib_Error;
    sDescription = "Cannot retrieve logging config !";
    
    sDpeExtension = sFullDpeName;

    string deviceName = unGenericDpFunctions_getDpName(sFullDpeName);
    unGenericDpFunctions_getParameters(deviceName, dsParameters);

    //Location
    dynClear(dsSplit);
    string sSystemName = getSystemName();
    dsSplit = strsplit(sSystemName, "_");
    if (dynlen(dsSplit) > 1) {
        sLocation = dsSplit[1];
    } else {
        sLocation = substr(sSystemName, 0, strlen(sSystemName) - 1);
    }

    //Application
    dynClear(dsSplit);
    dsSplit = strsplit(sFullDpeName, "-");
    if (dynlen(dsSplit) > 3) {
        sApplication = dsSplit[3];
    } else {
        sApplication = "UNKNOWN";
    }

    //Nature
    if (strpos(dsParameters[UN_PARAMETER_NATURE], ",") >= 0) { //cas multinature
        dyn_string dsT1 = strsplit(dsParameters[UN_PARAMETER_NATURE], ",");
        int i = 1;
        sNature = "";
        while (sNature == "") {
            sNature = dsT1[i];
            i++;
        }
    } else { sNature = dsParameters[UN_PARAMETER_NATURE]; }

    //Domain
    if (strpos(dsParameters[UN_PARAMETER_DOMAIN], ",") >= 0) { //cas multidomaine
        dyn_string dsT1 = strsplit(dsParameters[UN_PARAMETER_DOMAIN], ",");
        int i = 1;
        sDomaine = "";
        while (sDomaine == "") {
            sDomaine = dsT1[i];
            i++;
        }
    } else {
        sDomaine = dsParameters[UN_PARAMETER_DOMAIN];
    }

    dynClear(dsSplit);
    dsSplit = strsplit(sFullDpeName, ".");
    int iLen = dynlen(dsSplit);
    if (iLen == 3) {	
        // UNICOS Device
        if (dsSplit[2] == "ProcessInput") {            
            
            sDescription = unGenericDpFunctions_getDescription(deviceName);
            sDpAlias = dpGetAlias(dsSplit[1] + ".");
            sDpeExtension = substr(sFullDpeName, strpos(sFullDpeName, ".") + 1, strlen(sFullDpeName));
            sName = sDpAlias + "." + dsSplit[3];

            if (sLocation == "") {
                DebugN("ERROR: Location is empty for " + deviceName);
                return;
            }
            if (sApplication == "") {
                DebugN("ERROR: Application is empty for " + deviceName);
                return;
            }
            if (sNature == "") {
                DebugN("ERROR: Nature is empty for " + deviceName);
                return;
            }
            if (sDomaine == "") {
                DebugN("ERROR: Domain is empty for " + deviceName);
                return;
            }         
                       
            if (patternMatch("*SPARE*", strtoupper(sDescription))) {
                sHierarchy = cs_unLHCLoggingDBUserLib_Ignore;
            } else {
                sHierarchy = cs_unLHCLoggingDBUserLib_Activity + "/" + sLocation + "/" + sApplication + "/" + sNature + "/" + sDomaine;    //Unicos hierarchy 
            }
            
        } else {
            DebugN("ERROR: 'ProcessInput' pattern not found !");
        }
    } else if (iLen == 2) {		
        // System Alarm ?
        unLHCLoggingDBUserLib_getSystemAlarmInfo(dsSplit, sLocation, sDpeExtension, sDescription, sDpAlias, sName, sHierarchy);
    } else { 
        // nothing to do
    }
}

/**
This is the default function for retrieving
logging config data for system alarms.

@param dsDpe            input, splitted full dpe name
@param sLocation        input, location string

@param sDpe             output, full name of the sytem alarm dpe
@param sDescription     output, description of the sytem alarm dpe
@param sAlias           output, empty string
@param sName            output, alarm name
@param sHierarchy       output, activity/location/PVSS_INTEGRITY

*/
public void unLHCLoggingDBUserLib_getSystemAlarmInfo(
    const dyn_string dsDpe,
    const string sLocation,
    string &sDpe,
    string &sDescription,
    string &sAlias,
    string &sName,
    string &sHierarchy)
{
    string sAlarm;
    int iPos, iPosErr, iPosCom, iPosTime, iPosunPLC, iPosS7, iPosGTW, iPosFE;

    iPos = strpos(dsDpe[1], c_unSystemAlarm_dpPattern);

    if (dsDpe[2] == "alarm" && iPos >= 0 && sLocation != "") {
        sAlarm = substr(dsDpe[1], iPos + strlen(c_unSystemAlarm_dpPattern));

        iPosErr = strpos(sAlarm, PLC_DS_Error_pattern);
        iPosCom = strpos(sAlarm, DS_pattern);
        iPosTime = strpos(sAlarm, DS_Time_pattern);
        iPosFE = strpos(sAlarm, FE_pattern);

        //for the moment only FE alarms
        if (iPosErr >= 0 || iPosCom >= 0 || iPosTime >= 0 || iPosFE >= 0) {
            if (iPosErr >= 0) {
                sAlarm = substr(sAlarm, iPosErr + strlen(PLC_DS_Error_pattern)) + ":" + "ERROR";
            } else if (iPosCom >= 0) {
                sAlarm = substr(sAlarm, iPosCom + strlen(DS_pattern)) + ":" + "COMMUNICATION";
            } else if (iPosTime >= 0) {
                sAlarm = substr(sAlarm, iPosTime + strlen(DS_Time_pattern)) + ":" + "TIME";
            } else if (iPosFE >= 0) {
                sAlarm = "";
            } else {
                sAlarm = "";
            }

            //Type of FE?
            if (sAlarm != "") {
                iPosunPLC = strpos(sAlarm, c_unSystemIntegrity_UnPlc);
                iPosS7 = strpos(sAlarm, c_unSystemIntegrity_S7);
                iPosGTW = strpos(sAlarm, DS_Time_pattern);
                int iPosCMW = strpos(sAlarm, "CMW_"); // c_unSystemIntegrity_CMW
                int iPosSOFT = strpos(sAlarm, "SOFT_FE_");

                if (iPosunPLC >= 0) {
                    sAlarm = substr(sAlarm, iPosunPLC + strlen(c_unSystemIntegrity_UnPlc));
                } else if (iPosS7 >= 0) {
                    sAlarm = substr(sAlarm, iPosS7 + strlen(c_unSystemIntegrity_S7));
                } else if (iPosGTW >= 0) {
                    sAlarm = substr(sAlarm, iPosGTW + strlen(c_unSystemIntegrity_S7));    //!!!!!!!!! cst has to be replaced
                } else if (iPosCMW >= 0) {
                    sAlarm = substr(sAlarm, iPosCMW + strlen("CMW_"));
                } else if (iPosSOFT >= 0) {
                    sAlarm = substr(sAlarm, iPosCMW + strlen("SOFT_FE_"));
                } else {
                    sAlarm = "";
                }
            }
        } else {
            sAlarm = "";
        }
        
        if (sAlarm != "") {
            sDpe = dsDpe[1] + "." + dsDpe[2];
            sDescription = unGenericDpFunctions_getDescription(sDpe);
            sName = sAlarm;		//FE NAME:ERROR TYPE
            sAlias = "";
            sHierarchy = cs_unLHCLoggingDBUserLib_Activity + "/" + sLocation + "/" + "PVSS_INTEGRITY";                 
        }    
        if (strpos(sAlarm, "RCP_FE") >= 0) {
             sHierarchy = "IGNORE";
        }
    } 
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////// DEPRECATED FUNCTIONS /////////////////////////////////////////////////////
/////////////////////////////////////// (should be implemented into their components later) //////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// FUNCTION:: unUserlib_COLLIMATOR
// Author: E. Blanco AB/CO IS   [05/08/2008]
// REquirements: Use the existing hiearchy in DB Logging
// Analog Inputs / Analog Parameters:
//     Header:  (LHC or Ti)/Collimators/Temperatures  (LHC where Domain[2]=IPx, Tx where Domain[2]=TI2,TI8)
//                                          Beam = (Domain[1])
//                                             IP = (Domain[2])
//                                                family = (Domain[3])
//
//  DO
//     Header:  (LHC or Ti)/Collimators/Temperatures  (LHC where Domain[1]=IPx, Tx where Domain[1]=TI2,TI8)
//                                          BIC
//
public void unUserlib_COLLIMATOR_Deprecated(string sDpe, string &sN,  string &sD, string &sH, string &sA, string &sE)
{
    dyn_string dsParameters, dsSplit;
    string sAlias, deviceName, sSystemName, sLocation, sApplication, sDomaine, sNature;
    bool bOK;
    int iLen;

    string sActivityCOLLIMATOR = "LHC";      // It can be LHC, TI2, TI8
    string sApplicationCOLLIMATOR = "Collimators";
    string sBICInputs = "BIC";
    string sCatTemps = "Temperatures";

    dyn_string dsNatureCOLLIMATOR;
    dyn_string dsDomaineCOLLIMATOR;
    string sDevType;

    sSystemName = getSystemName();
    sH = "ERROR";
    sE = sDpe;

    // DebugN("FUNCTION (5): unUserlib_COLLIMATOR_logging(sDpe= "+sDpe+ ")");

    //Domain / nature
    deviceName = unGenericDpFunctions_getDpName(sDpe);
    unGenericDpFunctions_getParameters(deviceName, dsParameters);

    //Nature
    if (strpos(dsParameters[UN_PARAMETER_NATURE], ",") >= 0) {		//cas multinature	Analog Input
        dsNatureCOLLIMATOR = strsplit(dsParameters[UN_PARAMETER_NATURE], ",");
    } else {
        dsNatureCOLLIMATOR[1] = dsParameters[UN_PARAMETER_NATURE];
    }
    // DebugN("dsNatureCOLLIMATOR= "+dsNatureCOLLIMATOR);
    //Domain
    if (strpos(dsParameters[UN_PARAMETER_DOMAIN], ",") >= 0) { //cas multidomaine
        dsDomaineCOLLIMATOR = strsplit(dsParameters[UN_PARAMETER_DOMAIN], ",");
    } else {
        dsDomaineCOLLIMATOR[1] = dsParameters[UN_PARAMETER_DOMAIN];
    }
    // DebugN("dsDomaineCOLLIMATOR= "+dsDomaineCOLLIMATOR);

    // survey_1:un-CFC_SR8_GT8EF-SURVEY-AnalogInput-00019.ProcessInput.PosSt
    dynClear(dsSplit);
    dsSplit = strsplit(sDpe, ".");
    iLen = dynlen(dsSplit);
    if (iLen == 3) {	//UNICOS Device
        // DebugN("UNICOS device: dsSplit= "+dsSplit);

        if (dsSplit[2] == "ProcessInput" && sApplicationCOLLIMATOR != "") {
            sAlias = dpGetAlias(dsSplit[1] + ".");
                    
            sA = sAlias;
            sE = substr(sDpe, strpos(sDpe, ".") + 1, strlen(sDpe));
       
            // Change _ by :
            // DebugN("sAlias"+sAlias);
            sN = sAlias;

            // DebugN("sA,sE,sN",sA,sE,sN);

            sDevType = dpTypeName(sDpe);
            // DebugN("sDevType= "+sDevType);

            if ((sDevType == "AnalogInput") || (sDevType == "AnalogParameter")) {
                // Adapt the field "sActivityCOLLIMATOR"
                if (strpos(dsDomaineCOLLIMATOR[2], "IP") >= 0) {
                    sActivityCOLLIMATOR = "LHC";
                    sH = sActivityCOLLIMATOR + "/" + sApplicationCOLLIMATOR + "/" + sCatTemps + "/" + dsDomaineCOLLIMATOR[1] + "/" + dsDomaineCOLLIMATOR[2] + "/" + dsDomaineCOLLIMATOR[3];		//COLLIMATOR hierarchy
                } else if (strpos(dsDomaineCOLLIMATOR[2], "TI") >= 0) {
                    sActivityCOLLIMATOR = dsDomaineCOLLIMATOR[2];
                    sH = sActivityCOLLIMATOR + "/" + sApplicationCOLLIMATOR + "/" + sCatTemps + "/" + dsDomaineCOLLIMATOR[3];		//COLLIMATOR hierarchy
                }

            } else if ((sDevType == "DigitalOutput") || (sDevType == "Alarm")) {
                // Adapt the field "sActivityCOLLIMATOR"
                if (strpos(dsDomaineCOLLIMATOR[1], "IP") >= 0) {
                    sActivityCOLLIMATOR = "LHC";
                    sH = sActivityCOLLIMATOR + "/" + sApplicationCOLLIMATOR + "/" + sCatTemps + "/" + sBICInputs + "/" + dsDomaineCOLLIMATOR[1] + "/" + sDevType;		//COLLIMATOR
                    DebugN("dsDomaineCOLLIMATOR[1]=" + dsDomaineCOLLIMATOR[1]);
                    DebugN("sH=", sH);
                } else if (strpos(dsDomaineCOLLIMATOR[1], "TI") >= 0) {
                    sActivityCOLLIMATOR = dsDomaineCOLLIMATOR[1];
                    sH = sActivityCOLLIMATOR + "/" + sApplicationCOLLIMATOR + "/" + sCatTemps + "/" + sBICInputs + "/" + sDevType;		//COLLIMATOR
                }
            } else {
                sH = "";
            }

            // DebugN("sH",sH);
            sD = unGenericDpFunctions_getDescription(deviceName);
            // DebugN("sD",sD);
        } else {
            DebugN("***************** ERROR ****************************");
        }
    } else if (iLen == 2) {		//System Alarm ?
        unUserlib_COLLIMATOR_SystemAlarmInfo_Deprecated(dsSplit, "Integrity", sE, sD, sA, sN, sH);
    }

    // DebugN("sN,sD,sH,sA,sE",sN,sD,sH,sA,sE);
}
//-----------------------------------------------------------------------------------------------------------
// Function: unUserlib_CRYOSector
// 		1st approach to the logging configuration
// 		Initially the simplest possible following the current storage method
//
public void unUserlib_CRYOSector_Deprecated(string sDpe, string &sN,  string &sD, string &sH, string &sA, string &sE)
{
    dyn_string dsParameters, dsSplit;
    string sAlias, deviceName, sSystemName, sLocation, sApplication, sDomaine, sNature;
    bool bOK;
    int iLen;
    string sDevType;
    string dpeAlias;

    // DebugN("FUNCTION:: unUserlib_CRYOSector(sDpe= "+sDpe);

    sSystemName = getSystemName();
    sH = "ERROR";
    sE = sDpe;


    //Domain / nature
    deviceName = unGenericDpFunctions_getDpName(sDpe);
    unGenericDpFunctions_getParameters(deviceName, dsParameters);

    //Location
    dynClear(dsSplit);
    dsSplit = strsplit(sSystemName, "_");
    if (dynlen(dsSplit) > 1) {
        sLocation = dsSplit[1];
    } else {
        sLocation = substr(sSystemName, 0, strlen(sSystemName) - 1);
    }

    //Application
    dynClear(dsSplit);
    dsSplit = strsplit(sDpe, "-");
    if (dynlen(dsSplit) > 3) {
        sApplication = dsSplit[3];
    } else {
        sApplication = "UNKNOWN";
    }

    //Nature
    sDevType = dpTypeName(sDpe);
    switch (sDevType) {
        case "AnalogInput":
            sNature = "AI";
            break;
        case "DigitalInput":
            sNature = "DI";
            break;
        case "AnalogOutput":
            sNature = "AO";
            break;
        case "DigitalOutput":
            sNature = "DO";
            break;
        case "OnOff":
            sNature = "ONOFF";
            break;
        case "AnaDig":
            sNature = "ANADIG";
            break;
        case "Analog":
            sNature = "ANALOG";
            break;
        case "Controller":
            sNature = "PID";
            break;
        case "ProcessControlObject":
            sNature = "PCO";
            break;
        case "Alarm":
            sNature = "ALARM";
            break;
        case "AnalogParameter":
            sNature = "APAR";
            break;
        case "WordParameter":
            sNature = "WPAR";
            break;
        case "DigitalParameter":
            sNature = "DPAR";
            break;
        case "AnalogStatus":
            sNature = "AS";
            break;
        case "WordStatus":
            sNature = "WS";
            break;
        case "_UnSystemAlarm":
            break;
        default:
            DebugN("*** ERROR Nature: dpe= " + sDpe + "type UNKNOWN: <" + sDevType + ">");
    }
    /*
    if(strpos(dsParameters[UN_PARAMETER_NATURE], ",") >=0) //cas multinature
    	{
    		dyn_string dsT1 = strsplit(dsParameters[UN_PARAMETER_NATURE], ",");
    		int i=1;
    		sNature = "";
    		while (sNature =="")
    			{
    			sNature = dsT1[i];
    			i++;
    			}
    	}
    else 	sNature = dsParameters[UN_PARAMETER_NATURE];
    */

    //Domain

    sDevType = dpTypeName(sDpe);
    switch (sDevType) {
        // single domain
        case "AnalogInput":
        case "AnalogOutput":
        case "DigitalInput":
        case "DigitalOutput":
        case "AnaDig":
        case "OnOff":
        case "Analog":
        case "Controller":
        case "ProcessControlObject":
        case "Alarm":
        case "AnalogParameter":
        case "DigitalParameter":
        case "WordParameter":
        case "WordStatus":
        case "AnalogStatus":
            // sDomaine = dsParameters[UN_PARAMETER_DOMAIN];
            // Empty Domain ??
            // Always take the LOCATION of the DEVICE as the DOMAIN.


            dpeAlias = unGenericDpFunctions_getAlias(deviceName);
            dynClear(dsSplit);
            dsSplit = strsplit(dpeAlias, "_");	// eq_loc_name
            sDomaine = dsSplit[2];
        case "_UnSystemAlarm":
            break;
        default:
            DebugN("*** ERROR Domain: dpe= " + sDpe + "type UNKNOWN: <" + sDevType + ">");
    }

    /*
    if(strpos(dsParameters[UN_PARAMETER_DOMAIN], ",") >=0) //cas multidomaine
    	{
    		dyn_string dsT1 = strsplit(dsParameters[UN_PARAMETER_DOMAIN], ",");
    		int i=1;
    		sDomaine = "";
    		while (sDomaine =="")
    			{
    			sDomaine = dsT1[i];
    			i++;
    			}
    	}
    else
    	sDomaine = dsParameters[UN_PARAMETER_DOMAIN];
    */

    // other...

    dynClear(dsSplit);
    dsSplit = strsplit(sDpe, ".");
    iLen = dynlen(dsSplit);
    if (iLen == 3) {	//UNICOS Device
        if (dsSplit[2] == "ProcessInput") {		// && sLocation!="" && sApplication!="" && sNature!="" && sDomaine!="")
            sAlias = dpGetAlias(dsSplit[1] + ".");

            sA = sAlias;
            sE = substr(sDpe, strpos(sDpe, ".") + 1, strlen(sDpe));

            sN = sAlias + "." + dsSplit[3];
            sD = unGenericDpFunctions_getDescription(deviceName);

            bOK = TRUE;
            if (sLocation == "") {
                bOK = FALSE;
                DebugN("ERROR: Location is empty for " + deviceName);
            } else {
                if (sApplication == "") {
                    bOK = FALSE;
                    DebugN("ERROR: Application is empty for " + deviceName);
                } else {
                    if (sNature == "") {
                        bOK = FALSE;
                        DebugN("ERROR: Nature is empty for " + deviceName);
                    } else {
                        if (sDomaine == "") {
                            bOK = FALSE;
                            DebugN("ERROR: Domain is empty for " + deviceName);
                        }
                    }
                }
            }

            if (bOK) {
                sH = cs_unLHCLoggingDBUserLib_Activity + "/" + sLocation + "/" + sApplication + "/" + sNature + "/" + sDomaine;    //Unicos hierarchy
            }

        } else {
            DebugN("Error: \"ProcessInput\" pattern not found!");
        }
    } else if (iLen == 2) {		//System Alarm ?
        DebugN("SYSTEM ALARM: dsSplit=" + dsSplit);
        unLHCLoggingDBUserLib_getSystemAlarmInfo(dsSplit, sLocation, sE, sD, sA, sN, sH);
    }

}

//-----------------------------------------------------------------------------------------------------------
// Function: unUserlib_CIET
// 		1st approach to the logging configuration
// 		Initially the simplest possible following the current storage method
//
public void unUserlib_CIET_Deprecated(string sDpe, string &sN,  string &sD, string &sH, string &sA, string &sE)
{
    dyn_string dsParameters, dsSplit;
    string sAlias, deviceName, sSystemName, sLocation, sApplication, sDomaine, sNature;
    bool bOK;
    int iLen;
    string sDevType;


    //DebugN("FUNCTION:: unUserlib_CIET(sDpe= "+sDpe);

    sSystemName = getSystemName();
    sH = "ERROR";
    sE = sDpe;

    
    //Domain / nature
    deviceName = unGenericDpFunctions_getDpName(sDpe);
    unGenericDpFunctions_getParameters(deviceName, dsParameters);

    //Location
    dynClear(dsSplit);
    dsSplit = strsplit(sSystemName, "_");
    if (dynlen(dsSplit) > 1) {
        sLocation = dsSplit[1];
    } else {
        sLocation = substr(sSystemName, 0, strlen(sSystemName) - 1);
    }

    //Application
    dynClear(dsSplit);
    dsSplit = strsplit(sDpe, "-");
    if (dynlen(dsSplit) > 3) {
        sApplication = dsSplit[3];
    } else {
        sApplication = "UNKNOWN";
    }

    //Nature
    sDevType = dpTypeName(sDpe);


    //Nature
    if (dpExists(sDpe)) {
        sDevType = dpTypeName(sDpe);
    }

    if (!dpExists(sDpe)) {
        DebugN("-----");
        DebugN("*** ERROR DPE does not exists -->  sDpe= " + sDpe + "  //  " + "sDevType = " + sDevType);
    }


    switch (sDevType) {
        case "TT":
            sNature = "TT";
            break;
        case "PT":
            sNature = "PT";
            break;
        case "LT":
            sNature = "LT";
            break;
        case "DI":
            sNature = "DI";
            break;
        case "EH":
            sNature = "EH";
            break;
        case "EHBS":
            sNature = "EHBS";
            break;
        case "WFIPEvenAgent":
            sNature = "WFIPAgent";
            break;
        case "WFIPOddAgent":
            sNature = "WFIPAgent";
            break;
        case "WFIPSegment":
            sNature = "WFIPSegment";
            break;
        default:
            DebugN("*** ERROR Nature -->  dpe= " + sDpe + "type UNKNOWN: <" + sDevType + ">");
    }
    //DebugN("deviceName= "+deviceName+" sNature="+sNature+" sLocation="+sLocation+ " sApplication="+sApplication);

    //Domain

    //sDevType=dpTypeName(sDpe);

    switch (sDevType) {
        // It could multiple domain
        case "TT":
        case "PT":
        case "LT":
        case "EH":
        case "EHBS":

            //cas multidomaine
            if (strpos(dsParameters[UN_PARAMETER_DOMAIN], ",") >= 0) {
                dyn_string dsT1 = strsplit(dsParameters[UN_PARAMETER_DOMAIN], ",");
                sDomaine = dsT1[3];
            } else {
                sDomaine = dsParameters[UN_PARAMETER_DOMAIN];
            }
            break;


        // single domain
        case "DI":
        case "WFIPEvenAgent":
        case "WFIPOddAgent":
        case "WFIPSegment":
            sDomaine = dsParameters[UN_PARAMETER_DOMAIN];
            break;
        default:
            DebugN("*** ERROR Domain -->  dpe= " + sDpe + "type UNKNOWN: <" + sDevType + ">");
    }

    //DebugN("deviceName= "+deviceName+" sDomaine="+sDomaine);
    // other...

    dynClear(dsSplit);
    dsSplit = strsplit(sDpe, ".");
    iLen = dynlen(dsSplit);
    if (iLen == 4) {    //CIET Device
        if (dsSplit[2] == "ProcessInput") {     // && sLocation!="" && sApplication!="" && sNature!="" && sDomaine!="")
            sAlias = dpGetAlias(dsSplit[1] + ".");

            sA = sAlias;
            sE = substr(sDpe, strpos(sDpe, ".") + 1, strlen(sDpe));

            sN = sAlias + "." + dsSplit[4];
            // THCOUPLETEMPERATURECALC too long -> THCPTEMPCALC
            if (patternMatch("*thCoupleTemperatureCalc*", sN)) {
                strreplace(sN, "thCoupleTemperatureCalc", "thCpTempCalc");
            }

            sD = unGenericDpFunctions_getDescription(deviceName);

            bOK = TRUE;
            if (sLocation == "") {
                bOK = FALSE;
                DebugN("ERROR: Location is empty for " + deviceName);
            } else {
                if (sApplication == "") {
                    bOK = FALSE;
                    DebugN("ERROR: Application is empty for " + deviceName);
                } else {
                    if (sNature == "") {
                        bOK = FALSE;
                        DebugN("ERROR: Nature is empty for " + deviceName);
                    } else {
                        if (sDomaine == "") {
                            bOK = FALSE;
                            DebugN("ERROR: Domain is empty for " + deviceName);
                        }
                    }
                }
            }

            if (bOK) {
                sH = cs_unLHCLoggingDBUserLib_Activity + "/" + sLocation + "/" + sApplication + "/" + sNature + "/" + sDomaine;    //Unicos hierarchy
            }

        } else {
            DebugN("Error: \"ProcessInput\" pattern not found!");
        }
    }

    else if (iLen == 2) {
        //System Alarm ?
        dynClear(dsSplit);
        dsSplit = strsplit(sDpe, ".");
        if (dsSplit[2] == "ST_GTWDEV")
            // Treat: sDpe= un-CFC_SH8_QL1IJ__Z-RM_UJ76-EH-00001.ST_GTWDEV
        {
            //DebugN("************* ST_GTWDEV *****************");
            sAlias = dpGetAlias(dsSplit[1] + ".");
            sA = sAlias;
            sE = substr(sDpe, strpos(sDpe, ".") + 1, strlen(sDpe));
            sN = sAlias + "." + dsSplit[2];
            sD = unGenericDpFunctions_getDescription(deviceName);
            sH = cs_unLHCLoggingDBUserLib_Activity + "/" + sLocation + "/" + sApplication + "/" + sNature + "/" + sDomaine;        //Unicos hierarchy
        } else {
            unLHCLoggingDBUserLib_getSystemAlarmInfo(dsSplit, sLocation, sE, sD, sA, sN, sH);
        }
    }

//  DebugN("sN,sD,sH,sA,sE",sN,sD,sH,sA,sE);
}

//-----------------------------------------------------------------------------------------------------------

public void unUserlib_ECR_Logging_Deprecated(string sDpe, string &sN,  string &sD, string &sH, string &sA, string &sE)
{
    dyn_string dsParameters, dsSplit;
    string deviceName, sSystemName, sLocation, sApplication, sDomaine, sNature, sAppli;
    int iLen;

    sH = "ERROR";
    sE = sDpe;


    //Location: ATLAS or CMS
    dynClear(dsSplit);
    sSystemName = getSystemName();
    dsSplit = strsplit(sSystemName, "_");
    if (dynlen(dsSplit) > 1) {
        if (dsSplit[1] == "P1") {
            sLocation = "ATLAS";
        } else if (dsSplit[1] == "P5") {
            sLocation = "CMS";
        }
    }

    deviceName = unGenericDpFunctions_getDpName(sDpe);
    unGenericDpFunctions_getParameters(deviceName, dsParameters);

    //Domaine
    if (strpos(dsParameters[UN_PARAMETER_DOMAIN], ",") >= 0) { //multidomains
        dyn_string dsT1 = strsplit(dsParameters[UN_PARAMETER_DOMAIN], ",");
        int i = 1;
        sDomaine = "";
        while (sDomaine == "") {
            sDomaine = dsT1[i];
            i++;
        }
    } else {
        sDomaine = dsParameters[UN_PARAMETER_DOMAIN];
    }

    //Application
    sAppli = unGenericDpFunctions_getApplication(deviceName);
    if (sAppli != "") {
        sApplication = unUserlib_ECR_Applications_Deprecated(sAppli);
    }

    //Nature
    if (strpos(dsParameters[UN_PARAMETER_NATURE], ",") >= 0) { //multinatures
        dyn_string dsT1 = strsplit(dsParameters[UN_PARAMETER_NATURE], ",");
        int i = 1;
        sNature = "";
        while (sNature == "") {
            sNature = dsT1[i];
            i++;
        }
    } else {
        sNature = dsParameters[UN_PARAMETER_NATURE];
    }

    dynClear(dsSplit);
    dsSplit = strsplit(sDpe, ".");
    iLen = dynlen(dsSplit);

    if (iLen == 3) {				//Devices
        if (dsSplit[2] == "ProcessInput" && sLocation != "" && sApplication != "" && sNature != "") {
            sA = dpGetAlias(dsSplit[1] + ".");
            sN = sA + "." + dsSplit[3];
            sE = substr(sDpe, strpos(sDpe, ".") + 1, strlen(sDpe));
            sH = cs_unLHCLoggingDBUserLib_Activity + "/" + sLocation + "/" + sApplication + "/" + sNature + "/" + sDomaine;
            sD = unGenericDpFunctions_getDescription(deviceName);
        } else {
            DebugN("Error: Domain= " + sDomaine + " Location= " + sLocation + " sAppli=" + sAppli + " Application= " + sApplication + " Nature= "  + sNature);
        }
    } else if (iLen == 2) {		//System Alarm ?
        unLHCLoggingDBUserLib_getSystemAlarmInfo(dsSplit, "ATLAS", sE, sD, sA, sN, sH);
    }

}

//-----------------------------------------------------------------------------------------------------------

private string unUserlib_ECR_Applications_Deprecated(const string sName)
{
    dyn_dyn_string ddsNames;
    dyn_string dsNames, dsAppli;
    int iPos, iLen, i;
    string sAppli;

    dsNames = makeDynString("MRCP", "MRCB", "SRCP", "SRCB", "PCSBT", "PCSSOL", "ANRSCP", "ANRSCB", "LARG", "QS_C", "QU_C", "PCSCL");
    iLen = dynlen(dsNames);

    //1
    iPos = dynContains(dsNames, "MRCP");
    if (iPos > 0) {
        dsAppli = makeDynString("QSC1H_A", "QSC1H", "QS_A");
        ddsNames[iPos] = dsAppli;
    }

    //2
    iPos = dynContains(dsNames, "MRCB");
    if (iPos > 0) {
        dsAppli = makeDynString("QUR1H_A", "QUR1H");
        ddsNames[iPos] = dsAppli;
    }

    //3
    iPos = dynContains(dsNames, "SRCP");
    if (iPos > 0) {
        dsAppli = makeDynString("QSC2H_A", "QSC2H");
        ddsNames[iPos] = dsAppli;
    }

    //4
    iPos = dynContains(dsNames, "SRCB");
    if (iPos > 0) {
        dsAppli = makeDynString("QUR2H_A", "QUI2H_A", "QUR2H");
        ddsNames[iPos] = dsAppli;
    }

    //5
    iPos = dynContains(dsNames, "PCSBT");
    if (iPos > 0) {
        dsAppli = makeDynString("QXPTH_A", "QXPTH", "BTPCS");
        ddsNames[iPos] = dsAppli;
    }

    //6
    iPos = dynContains(dsNames, "PCSSOL");
    if (iPos > 0) {
        dsAppli = makeDynString("QXZSH_A", "QXZSH", "QUZSH_A", "QUZSH", "SOLPCS");
        ddsNames[iPos] = dsAppli;
    }

    //7
    iPos = dynContains(dsNames, "ANRSCP");
    if (iPos > 0) {
        dsAppli = makeDynString("QSC1N_A", "QSCBN");
        ddsNames[iPos] = dsAppli;
    }

    //8
    iPos = dynContains(dsNames, "ANRSCB");
    if (iPos > 0) {
        dsAppli = makeDynString("QUR1N_A", "QUU1N_A", "QPRN");
        ddsNames[iPos] = dsAppli;
    }

    //9
    iPos = dynContains(dsNames, "LARG");
    if (iPos > 0) {
        dsAppli = makeDynString("QXD4N", "QXKAA", "QXKBA", "QXKCA");
        ddsNames[iPos] = dsAppli;
    }

    //10
    iPos = dynContains(dsNames, "QS_C");
    if (iPos > 0) {
        dsAppli = makeDynString("QSC1H_C");
        ddsNames[iPos] = dsAppli;
    }

    //11
    iPos = dynContains(dsNames, "QU_C");
    if (iPos > 0) {
        dsAppli = makeDynString("QUR1H_C", "QUK1H_C", "QUU1H_C", "QUM1H_C");
        ddsNames[iPos] = dsAppli;
    }

    //12
    iPos = dynContains(dsNames, "PCSCL");
    if (iPos > 0) {
        dsAppli = makeDynString("QUIX1A", "QUX1H", "CLPCS");
        ddsNames[iPos] = dsAppli;
    }

    for (i = 1; i <= iLen; i++) {
        if (dynContains(ddsNames[i], sName) > 0) {
            sAppli = dsNames[i];
            i = iLen;
        }
    }

    return sAppli;
}

//-----------------------------------------------------------------------------------------------------------

public void unUserlib_SURVEY_logging_Deprecated(string sDpe, string &sN,  string &sD, string &sH, string &sA, string &sE)
{
    dyn_string dsParameters, dsSplit;
    string sAlias, deviceName, sSystemName, sLocation, sApplication, sDomaine, sNature;
    int iLen;
    string sActivitySURVEY = "SURVEY";
    string sApplicationSURVEY = "LOW-BETA";
    dyn_string sNatureSURVEY;
    dyn_string sDomaineSURVEY;
    string sDevType;

    sSystemName = getSystemName();
    sH = "ERROR";
    sE = sDpe;

    // DebugN("FUNCTION (5): unUserlib_SURVEY_logging(sDpe= "+sDpe+ ")");

    // DebugN("sDpe= "+sDpe);


    //Domain / nature
    deviceName = unGenericDpFunctions_getDpName(sDpe);
    unGenericDpFunctions_getParameters(deviceName, dsParameters);

    //Nature
    if (strpos(dsParameters[UN_PARAMETER_NATURE], ",") >= 0) {		//cas multinature	Analog Input
        sNatureSURVEY = strsplit(dsParameters[UN_PARAMETER_NATURE], ",");
    } else {
        sNatureSURVEY[1] = dsParameters[UN_PARAMETER_NATURE];
    }
    // DebugN("sNatureSURVEY= "+sNatureSURVEY);
    //Domain
    if (strpos(dsParameters[UN_PARAMETER_DOMAIN], ",") >= 0) { //cas multidomaine
        sDomaineSURVEY = strsplit(dsParameters[UN_PARAMETER_DOMAIN], ",");
    } else {
        sDomaineSURVEY[1] = dsParameters[UN_PARAMETER_DOMAIN];
    }
    // DebugN("sDomaineSURVEY= "+sDomaineSURVEY);

    // survey_1:un-CFC_SR8_GT8EF-SURVEY-AnalogInput-00019.ProcessInput.PosSt
    dynClear(dsSplit);
    dsSplit = strsplit(sDpe, ".");
    iLen = dynlen(dsSplit);
    if (iLen == 3) {	//UNICOS Device
        // DebugN("UNICOS device: dsSplit= "+dsSplit);

        if (dsSplit[2] == "ProcessInput" && sActivitySURVEY != "" && sApplicationSURVEY != "") {
            sAlias = dpGetAlias(dsSplit[1] + ".");

            sA = sAlias;
            sE = substr(sDpe, strpos(sDpe, ".") + 1, strlen(sDpe));
   
            // Change _ by :
            // DebugN("sAlias"+sAlias);
            sN = sAlias;

            // AI
            strreplace(sN, ".STEINER_", ".STEINER:");
            strreplace(sN, "_WPS_", "_WPS:");
            strreplace(sN, "_LO_", "_LO:");
            strreplace(sN, "_HI_", "_HI:");
            strreplace(sN, "_DOMS_", "_DOMS:");	// new spec IP1
            strreplace(sN, ".SU3_", ".SU3:");		// new spec IP1,IP5: also in DI,DO
            // DI,DO
            strreplace(sN, ".SU_", ".SU:");
            strreplace(sN, ".SU1_", ".SU1:");
            strreplace(sN, ".SU2_", ".SU2:");
            strreplace(sN, "_DO_RESET", ":DO_RESET"); // new in spec IP1,IP5


            // DebugN("sA,sE,sN",sA,sE,sN);

            sDevType = dpTypeName(sDpe);
            // DebugN("sDevType= "+sDevType);

            if (sDevType == "AnalogInput") {
                sH = sActivitySURVEY + "/" + sApplicationSURVEY + "/" + sDomaineSURVEY[1] + "/" + sDomaineSURVEY[2] + "/" + sDomaineSURVEY[4] + "/" + sNatureSURVEY[1];    //Unicos hierarchy
            } else if ((sDevType == "DigitalInput") || (sDevType == "DigitalOutput")) {
                sH = sActivitySURVEY + "/" + sApplicationSURVEY + "/" + sDomaineSURVEY[1] + "/" + sDomaineSURVEY[2] + "/" + sDomaineSURVEY[3] + "/" + sNatureSURVEY[1];    //Unicos hierarchy
            }

            // DebugN("sH",sH);
            sD = unGenericDpFunctions_getDescription(deviceName);
            // DebugN("sD",sD);
        } else {
            DebugN("***************** ERROR ****************************");
        }
    } else if (iLen == 2) {		//System Alarm ?
        unUserlib_SURVEY_SystemAlarmInfo_Deprecated(dsSplit, "LOW-BETA", sE, sD, sA, sN, sH);
    }

    // DebugN("sN,sD,sH,sA,sE",sN,sD,sH,sA,sE);
}

//-----------------------------------------------------------------------------------------------------------

private void unUserlib_SURVEY_SystemAlarmInfo_Deprecated(const dyn_string dsDpe, const string sLoc, string &sDpe, string &sDescription, string &sAlias, string &sNane, string &sHierarchy)
{
    string sAlarm;
    string sActivitySURVEY = "SURVEY";

    int iPos, iPosErr, iPosCom, iPosTime, iPosunPLC, iPosS7, iPosGTW;

    iPos = strpos(dsDpe[1], c_unSystemAlarm_dpPattern);

    if (dsDpe[2] == "alarm" && iPos >= 0 && sLoc != "") {
        sAlarm = substr(dsDpe[1], iPos + strlen(c_unSystemAlarm_dpPattern));

        iPosErr = strpos(sAlarm, PLC_DS_Error_pattern);
        iPosCom = strpos(sAlarm, DS_pattern);
        iPosTime = strpos(sAlarm, DS_Time_pattern);

        //for the moment only FE alarms
        if (iPosErr >= 0 || iPosCom >= 0 || iPosTime >= 0) {
            if (iPosErr >= 0) {
                sAlarm = substr(sAlarm, iPosErr + strlen(PLC_DS_Error_pattern)) + ":" + "ERROR";
            } else if (iPosCom >= 0) {
                sAlarm = substr(sAlarm, iPosCom + strlen(DS_pattern)) + ":" + "COMMUNICATION";
            } else if (iPosTime >= 0) {
                sAlarm = substr(sAlarm, iPosTime + strlen(DS_Time_pattern)) + ":" + "TIME";
            } else {
                sAlarm = "";
            }

            //Type of FE?
            if (sAlarm != "") {
                iPosunPLC = strpos(sAlarm, c_unSystemIntegrity_UnPlc);
                iPosS7 = strpos(sAlarm, c_unSystemIntegrity_S7);
                iPosGTW = strpos(sAlarm, DS_Time_pattern);

                if (iPosunPLC >= 0) {
                    sAlarm = substr(sAlarm, iPosunPLC + strlen(c_unSystemIntegrity_UnPlc));
                } else if (iPosS7 >= 0) {
                    sAlarm = substr(sAlarm, iPosS7 + strlen(c_unSystemIntegrity_S7));
                } else if (iPosGTW >= 0) {
                    sAlarm = substr(sAlarm, iPosGTW + strlen(c_unSystemIntegrity_S7));    //!!!!!!!!! cst has to be replaced
                } else {
                    sAlarm = "";
                }
            }
        } else {
            sAlarm = "";
        }

        if (sAlarm != "") {
            sDpe = dsDpe[1] + "." + dsDpe[2];
            sDescription = unGenericDpFunctions_getDescription(sDpe);
            sNane = sAlarm;		//FE NAME:ERROR TYPE
            sAlias = "";
            sHierarchy = sActivitySURVEY + "/" + sLoc + "/" + "PVSS_INTEGRITY";
        }
    }
}
//-----------------------------------------------------------------------------------------------------------

private void unUserlib_COLLIMATOR_SystemAlarmInfo_Deprecated(const dyn_string dsDpe, const string sLoc, string &sDpe, string &sDescription, string &sAlias, string &sNane, string &sHierarchy)
{
    string sAlarm;
    string sActivitySURVEY = "LHC" + "/" + "Collimators" + "/" + "Temperatures";

    int iPos, iPosErr, iPosCom, iPosTime, iPosunPLC, iPosS7, iPosGTW;

    iPos = strpos(dsDpe[1], c_unSystemAlarm_dpPattern);

    if (dsDpe[2] == "alarm" && iPos >= 0 && sLoc != "") {
        sAlarm = substr(dsDpe[1], iPos + strlen(c_unSystemAlarm_dpPattern));

        iPosErr = strpos(sAlarm, PLC_DS_Error_pattern);
        iPosCom = strpos(sAlarm, DS_pattern);
        iPosTime = strpos(sAlarm, DS_Time_pattern);

        //for the moment only FE alarms
        if (iPosErr >= 0 || iPosCom >= 0 || iPosTime >= 0) {
            if (iPosErr >= 0) {
                sAlarm = substr(sAlarm, iPosErr + strlen(PLC_DS_Error_pattern)) + ":" + "ERROR";
            } else if (iPosCom >= 0) {
                sAlarm = substr(sAlarm, iPosCom + strlen(DS_pattern)) + ":" + "COMM";
            } else if (iPosTime >= 0) {
                sAlarm = substr(sAlarm, iPosTime + strlen(DS_Time_pattern)) + ":" + "TIME";
            } else {
                sAlarm = "";
            }

            //Type of FE?
            if (sAlarm != "") {
                iPosunPLC = strpos(sAlarm, c_unSystemIntegrity_UnPlc);
                iPosS7 = strpos(sAlarm, c_unSystemIntegrity_S7);
                iPosGTW = strpos(sAlarm, DS_Time_pattern);

                if (iPosunPLC >= 0) {
                    sAlarm = substr(sAlarm, iPosunPLC + strlen(c_unSystemIntegrity_UnPlc));
                } else if (iPosS7 >= 0) {
                    sAlarm = substr(sAlarm, iPosS7 + strlen(c_unSystemIntegrity_S7));
                } else if (iPosGTW >= 0) {
                    sAlarm = substr(sAlarm, iPosGTW + strlen(c_unSystemIntegrity_S7));    //!!!!!!!!! cst has to be replaced
                } else {
                    sAlarm = "";
                }
            }
        } else {
            sAlarm = "";
        }

        if (sAlarm != "") {
            sDpe = dsDpe[1] + "." + dsDpe[2];
            sDescription = unGenericDpFunctions_getDescription(sDpe);
            sNane = sAlarm;		//FE NAME:ERROR TYPE
            sAlias = "";
            sHierarchy = sActivitySURVEY + "/" + sLoc ;
        }
    }
}
//-----------------------------------------------------------------------------------------------------------

public void unUserlib_WICLHC_Logging_Deprecated(string sDpe, string &sN,  string &sD, string &sH, string &sA, string &sE)
{
    dyn_string dsSplit, dsSplitAlias;
    string sAlias, deviceName, sSystemName, sLocation, sApplication, sDescription;

    int iLen, iLenAlias;

    sSystemName = getSystemName();
    sH = "ERROR";
    sE = sDpe;

    //Location
    sLocation = "LHC";

    //Application
    sApplication = "Warm Interlocks";

    dsSplit = strsplit(sDpe, ".");
    iLen = dynlen(dsSplit);
    if (iLen == 3) {	//Device
        deviceName = unGenericDpFunctions_getDpName(sDpe);
        deviceName = unGenericDpFunctions_getAlias(deviceName);
        //deviceName=substr(deviceName, 0, strpos(deviceName, "/LHC"));
        sDescription = unGenericDpFunctions_getDescription(sDpe);
        sAlias = unGenericDpFunctions_getAlias(sDpe);

        dynClear(dsSplit);
        dsSplit = strsplit(sDescription, "/");
        dsSplitAlias = strsplit(sAlias, "/");
        iLen = dynlen(dsSplit);
        iLenAlias = dynlen(dsSplitAlias);

        if (iLen >= 2 && iLenAlias == 3) {					//	I/O || Monitoring
            sN = dsSplitAlias[1] + ":" + dsSplitAlias[2];
            sD = dsSplit[2] + dsSplit[1];
        } else {
            sN = "";
            sD = "Not defined";
        }

        sE = substr(sDpe, strpos(sDpe, ":") + 1);
        sH = sLocation + "/" + sApplication + "/" + deviceName;
    } else if (iLen == 2) {		//System Alarm ?
        unLHCLoggingDBUserLib_getSystemAlarmInfo(dsSplit, sLocation, sE, sD, sA, sN, sH);
        sH = sLocation + "/" + sApplication + "/PVSS_INTEGRITY";
    }
}

//-----------------------------------------------------------------------------------------------------------

public void unUserlib_CryoLabInfos_Deprecated(string sDpe, string &sN,  string &sD, string &sH, string &sA, string &sE)
{
    dyn_string dsParameters, dsSplit;
    string sAlias, deviceName, sSystemName, sLocation, sApplication, sNature;
    bool bOK;
    int iLen;

    sSystemName = getSystemName();
    sH = "ERROR";
    sE = sDpe;

    
    //Domain - nature
    deviceName = unGenericDpFunctions_getDpName(sDpe);
    unGenericDpFunctions_getParameters(deviceName, dsParameters);

    //Application
    dynClear(dsSplit);
    dsSplit = strsplit(sDpe, "-");
    if (dynlen(dsSplit) > 3) {
        sApplication = dsSplit[3];
        //Special part to define the HIERARCHY for CryoLab... harcoded
        if (sApplication == "QSR1H_M") {
            sLocation = "Cryolab";
            sApplication = "CL165CB";
        }
    } else {
        sApplication = "UNKNOWN";
    }

    //Nature
    if (strpos(dsParameters[UN_PARAMETER_NATURE], ",") >= 0) { //cas multinature
        dyn_string dsT1 = strsplit(dsParameters[UN_PARAMETER_NATURE], ",");
        int i = 1;
        sNature = "";
        while (sNature == "") {
            sNature = dsT1[i];
            i++;
        }
    } else { sNature = dsParameters[UN_PARAMETER_NATURE]; }

    dynClear(dsSplit);
    dsSplit = strsplit(sDpe, ".");
    iLen = dynlen(dsSplit);
    if (iLen == 3) {	//UNICOS Device
        if (dsSplit[2] == "ProcessInput") {		// && sLocation!="" && sApplication!="" && sNature!="")
            sAlias = dpGetAlias(dsSplit[1] + ".");

            sA = sAlias;
            sE = substr(sDpe, strpos(sDpe, ".") + 1, strlen(sDpe));

            sN = sAlias + "." + dsSplit[3];
            sD = unGenericDpFunctions_getDescription(deviceName);

            bOK = TRUE;

            if (sLocation == "") {
                bOK = FALSE;
                DebugN("ERROR: Location is empty for " + deviceName);
            } else {
                if (sApplication == "") {
                    bOK = FALSE;
                    DebugN("ERROR: Application is empty for " + deviceName);
                } else {
                    if (sNature == "") {
                        bOK = FALSE;
                        DebugN("ERROR: Nature is empty for " + deviceName);
                    }
                }
            }

            if (bOK) {
                sH = cs_unLHCLoggingDBUserLib_Activity + "/" + sLocation + "/" + sApplication + "/" + sNature;    //hierarchy
            }

        } else {
            DebugN("Error: \"ProcessInput\" pattern not found!");
        }
    } else if (iLen == 2) {		//System Alarm ?
        unLHCLoggingDBUserLib_getSystemAlarmInfo(dsSplit, sLocation, sE, sD, sA, sN, sH);
    }

}

//-----------------------------------------------------------------------------------------------------------

public void unUserlib_NA48_Deprecated(string sDpe, string &sN,  string &sD, string &sH, string &sA, string &sE)
{
    dyn_string dsParameters, dsSplit;
    string sAlias, deviceName, sSystemName, sLocation, sApplication, sNature;
    bool bOK;
    int iLen;

    sSystemName = getSystemName();
    sH = "ERROR";
    sE = sDpe;

    //Domain - nature
    deviceName = unGenericDpFunctions_getDpName(sDpe);
    unGenericDpFunctions_getParameters(deviceName, dsParameters);

    //Application
    dynClear(dsSplit);
    dsSplit = strsplit(sDpe, "-");
    if (dynlen(dsSplit) > 3) {
        sApplication = dsSplit[3];
        //Special part to define the HIERARCHY for NA48... harcoded
        if (sApplication == "NA48_QX_N") {
            sLocation = "NA62";
            sApplication = "NOT USED";
        }
    } else {
        sApplication = "UNKNOWN";
    }

    //Nature
    if (strpos(dsParameters[UN_PARAMETER_NATURE], ",") >= 0) { //cas multinature
        dyn_string dsT1 = strsplit(dsParameters[UN_PARAMETER_NATURE], ",");
        int i = 1;
        sNature = "";
        while (sNature == "") {
            sNature = dsT1[i];
            i++;
        }
    } else { sNature = dsParameters[UN_PARAMETER_NATURE]; }

    dynClear(dsSplit);
    dsSplit = strsplit(sDpe, ".");
    iLen = dynlen(dsSplit);
    if (iLen == 3) {	//UNICOS Device
        if (dsSplit[2] == "ProcessInput") {		// && sLocation!="" && sApplication!="" && sNature!="")
            sAlias = dpGetAlias(dsSplit[1] + ".");

            sA = sAlias;
            sE = substr(sDpe, strpos(sDpe, ".") + 1, strlen(sDpe));

            sN = sAlias + "." + dsSplit[3];
            sD = unGenericDpFunctions_getDescription(deviceName);

            bOK = TRUE;

            if (sLocation == "") {
                bOK = FALSE;
                DebugN("ERROR: Location is empty for " + deviceName);
            } else {
                if (sApplication == "") {
                    bOK = FALSE;
                    DebugN("ERROR: Application is empty for " + deviceName);
                } else {
                    if (sNature == "") {
                        bOK = FALSE;
                        DebugN("ERROR: Nature is empty for " + deviceName);
                    }
                }
            }

            if (bOK) {
                sH = cs_unLHCLoggingDBUserLib_Activity + "/" + sLocation + "/" + sNature;    //hierarchy
            }

        } else {
            DebugN("Error: \"ProcessInput\" pattern not found!");
        }
    } else if (iLen == 2) {		//System Alarm ?
        unLHCLoggingDBUserLib_getSystemAlarmInfo(dsSplit, sLocation, sE, sD, sA, sN, sH);
    }
}

/**
Function for vacuum instruments with vacuum hierarchy

@param sFullDpeName     input, full data point element name

@param sName            output, variable name
@param sDescription     output, description
@param sHierarchy       output, systemName/dpType
@param sDpAlias         output, alias
@param sDpeExtension    output, element extension

*/
public void unLHCLoggingDBUserLib_getDeviceInfo_VAC(
    string sFullDpeName,
    string &sName,
    string &sDescription,
    string &sHierarchy,
    string &sDpAlias,
    string &sDpeExtension)
{
    dyn_string dsParameters, dsSplit;
    string sLocation, sApplication, sDomaine, sNature;
   
    sHierarchy = cs_unLHCLoggingDBUserLib_Error;
    sDescription = "Cannot retrieve logging config !";
    
    sDpeExtension = sFullDpeName;

    string deviceName = unGenericDpFunctions_getDpName(sFullDpeName);
    unGenericDpFunctions_getParameters(deviceName, dsParameters);

    //Location
    dynClear(dsSplit);
    string sApplicationName; 
    int ret= dpGet("_unApplication.applicationName", sApplicationName);
    dsSplit = strsplit(sApplicationName, " ");
    if (dynlen(dsSplit) > 1) {
        sLocation = dsSplit[1];
    } else {
        sLocation = sApplicationName;
    }

    //Application
    dynClear(dsSplit);
    dsSplit = strsplit(sFullDpeName, "-");
    if (dynlen(dsSplit) > 3) {
        sApplication = dsSplit[3];
    } else {
        sApplication = "UNKNOWN";
    }

    //Nature
    if (strpos(dsParameters[UN_PARAMETER_NATURE], ",") >= 0) { //cas multinature
        dyn_string dsT1 = strsplit(dsParameters[UN_PARAMETER_NATURE], ",");
        int i = 1;
        sNature = "";
        while (sNature == "") {
            sNature = dsT1[i];
            i++;
        }
    } else { sNature = dsParameters[UN_PARAMETER_NATURE]; }

    //Domain
    if (strpos(dsParameters[UN_PARAMETER_DOMAIN], ",") >= 0) { //cas multidomaine
        dyn_string dsT1 = strsplit(dsParameters[UN_PARAMETER_DOMAIN], ",");
        int i = 1;
        sDomaine = "";
        while (sDomaine == "") {
            sDomaine = dsT1[i];
            i++;
        }
    } else {
        sDomaine = dsParameters[UN_PARAMETER_DOMAIN];
    }
    if (sDomaine == PROJ) {
        sDomaine = sLocation;
    }      

    dynClear(dsSplit);
    dsSplit = strsplit(sFullDpeName, ".");
    int iLen = dynlen(dsSplit);
    if (iLen == 3) {	
        // UNICOS Device
        if (dsSplit[2] == "ProcessInput") {            
            
            sDescription = unGenericDpFunctions_getDescription(deviceName);
            sDpAlias = dpGetAlias(dsSplit[1] + ".");
            sDpeExtension = substr(sFullDpeName, strpos(sFullDpeName, ".") + 1, strlen(sFullDpeName));
            sName = sDpAlias + "." + dsSplit[3];

            if (sLocation == "") {
                DebugN("ERROR: Location is empty for " + deviceName);
                return;
            }
            if (sApplication == "") {
                DebugN("ERROR: Application is empty for " + deviceName);
                return;
            }
            if (sNature == "") {
                DebugN("ERROR: Nature is empty for " + deviceName);
                return;
            }
            if (sDomaine == "") {
                DebugN("ERROR: Domain is empty for " + deviceName);
                return;
            }         
                       
            if (patternMatch("*SPARE*", strtoupper(sDescription))) {
                sHierarchy = cs_unLHCLoggingDBUserLib_Ignore;
            } else {
                sHierarchy = "VAC/" + sLocation + "/" + sDomaine + "/"+ sNature;    //VAC hierarchy
            }
            
        } else {
            DebugN("ERROR: 'ProcessInput' pattern not found !");
        }
    } else if (iLen == 2) {		
        // System Alarm ?
        unLHCLoggingDBUserLib_getSystemAlarmInfo(dsSplit, sLocation, sDpeExtension, sDescription, sDpAlias, sName, sHierarchy);
    } else { 
        // nothing to do
    }
}


