/**
  This file contains all the functions related with unVacTrending module. It consists of the following
  procedures:

  * Opening a new trend window
  * Focussing an existing trend window
  * Adding a trend to an existing window
  * Handling the new online value for a existing trend
  * Guessing the unit of a DPE that needs to be added to the trend module (To be implemented)
  * Removing a trend from a existing trend window (To be implemented)
  * Closing an existing trend window (To be implemented)

*/

/**
  The module name for the trending panel
*/
const string UNVACTRENDING_MODULE = "unVacTrending";

/**
  The default panel used when opening the trend window

  @todo Make it possible to overwrite this
*/
const string UNVACTRENDING_PANEL = "vision/vacPanels/unVacTrending.pnl";


/**
  The datatype guessed by the trending tool
*/
enum unVacTrending_ScaleType {
  UNITLESS     =  0 , // No unit - no scale
  MBARVAC      =  1, // Vacuum pressure like pirani, penning etc.- [mbar] - 1e3 to 1e-13
  KELVINCRYO   =  2, // Cryogenic temperature - [K] - 0 to 350
  DEGCMEDIUM   =  3, // Temperature like bakeout - [DegC] - 0 to 450
  AMPSLOW      =  4, // Current like ion pump - [Amps] - 1e-2 to 1e-11
  AMPSMEDIUM   =  5, // Current like gauge emission current - [Amps] - 0 to 100
  AMPSHIGH     =  6, // High current like magnet - [Amps] - 0 to 1e4
  VOLTVERYLOW  =  7, // Voltage like valve set point - [Volts] - 0 to 48
  VOLTLOW      =  8, // Voltage - [Volts] - 0 to 1000
  VOLTHIGH     =  9, // Voltage like power supply - [Volts] - 0 to 1e4
  PERCENTAGE   = 10, // Percentage - [%] - 0 to 100
  STATUSOBJECT = 11, // Status (integer) - [State] - 0 to 15
  GEVENERGY    = 12, // Beam energy - [GeV] - 0 to 10
  MABEAM       = 13, // Beam current - [mA] - 0 to 1000
  BIT          = 14  // Bit state - 0 or 1
};

/**
  The unit type constants used for guessing the scaling

  @todo This might be improved to using regex, to make it more robust

*/
const string UNVACTRENDING_UNIT_MBAR = "mbar";
const string UNVACTRENDING_UNIT_VOLT = "V";
const string UNVACTRENDING_UNIT_VOLT_ALTERNATIVE = "Volt";
const string UNVACTRENDING_UNIT_AMPERE = "A";
const string UNVACTRENDING_UNIT_AMPERE_ALTERNATIVE = "Amps";

/**
  @brief Add a DPE to the trend graph by alias. By using the sModule parameter multiple trends can be chosen
  @param sDpeAlias The alias of the DPE, this is used to load the full name
  @param sLabel The label displayed in the legend of the trend, if not supplied the alias is automatically loaded
  @param sModule The module name to which the trend should be appended, if none is chosen the default trend is selected
*/
unVacTrending_addByAlias(string sDpeAlias, string sLabel = "", string sModule = UNVACTRENDING_MODULE){
  // Detect wheter the label is empty, if so add the alias as label
  if(sLabel == ""){
    sLabel = sDpeAlias;
  }
  // Determine dpe full name
  string sDpeFullName = unGenericDpFunctions_dpAliasToName(sDpeAlias);

  string dpeSystFullName = dpSubStr(sDpeFullName, DPSUB_SYS_DP_EL);
  // Add the dpe to the trend
  unVacTrending_add(dpeSystFullName, sLabel, sModule);
  //dpConnect requested by the EWO
  //dpConnect("unVacTrending_newOnlineValueCB", dpeSystFullName);
}



/**
  @brief Add a DPE to the trend graph by full name. By using the sModule parameter multiple trends can be chosen
  @param sDpeFullName The alias of the DPE, this is used to load the full name
  @param sLabel The label displayed in the legend of the trend, if not supplied the alias is automatically loaded
  @param sModule The module name to which the trend should be appended, if none is chosen the default trend is selected
*/
unVacTrending_add(string sDpeFullName, string sLabel = "", string sModule = UNVACTRENDING_MODULE){
  if(!isModuleOpen(sModule)){
    throw(makeError("unVacTrending", PRIO_WARNING, ERR_CONTROL, 1, "Module "+sModule+" is not opened, so trend cannot be added"));
  }

  // Detect wheter the label is empty, if so add the alias as label
  if(sLabel == ""){
    sLabel = dpGetAlias(sDpeFullName);
  }

  // Guess scaling and axes type based
  unVacTrending_ScaleType scaleType = unVacTrending_guessScaleType(sDpeFullName);

  // Get maximum time range in second according to the timestamp of the oldest value archived
  int secTrendRange = unVacTrending_getDpArchiveMaxTimeRange(sDpeFullName);

  // Add the dpe to the trend
  invokeMethod(sModule+"."+sModule+":unVacTrendingEwo_ewo1", "addTrend", sDpeFullName, sLabel, secTrendRange, (int) scaleType);

  // dpConnect shall be done in the panel!!!
  // Add dpe connect for online updating of the values
  // dpConnect("unVacTrending_newOnlineValueCB", sDpeFullName);

  // Handle errors from dpe connect
  dyn_errClass e = getLastError();
  if(dynlen(e) > 0)
  {
    throwError(e);
  }

}

/**
  @brief Open or focuse the specified trend module. If none is specified the default is opened or focussed
  @param sModule The trend module to open/focus, defaults to UNVACTRENDING_MODULE
  @return True if the panels has opened with succes, false if no focus could be acquired
*/
bool unVacTrending_open(string sModule = UNVACTRENDING_MODULE){
  bool isOpen = isModuleOpen(sModule);

  if(isOpen){
    restorePanel(sModule);
    int ret = stayOnTop(TRUE, sModule);
    ret= ret+stayOnTop(FALSE, sModule);
    return ret == 0;
  } else {
    ModuleOnWithPanel(sModule, 0, 150, 0, 0, 0, 0, 0,UNVACTRENDING_PANEL , sModule, makeDynString());
  }
  return true;
}


/**
  @brief Callback for updating the online value of the trend
  @param sDpName The DPE name for which the new value has arrived
  @param fValue The new value itself
*/
unVacTrending_newOnlineValueCB(string sDpName, float fValue){
  string sModule = UNVACTRENDING_MODULE;
  DebugTN("newOnlineValueCB : sModule unVacTrending :", sModule);
  DebugTN("newOnlineValueCB : dpName = ", sDpName, "; value = ", fValue);

  dyn_string nameSplit = strsplit(sDpName, ":");
  DebugTN("nameSplit = ", nameSplit);
  string dp;
  if (dynlen(nameSplit)==3){ // systName:dpeName:attr
    dp = nameSplit[1] + ":" + nameSplit[2];
  }
  else {
    dp = sDpName;
  }
  DebugTN("used name for ewo: ", dp, fValue);

  // Update the value in the trend
  invokeMethod(sModule+"."+sModule+":unVacTrendingEwo_ewo1", "onlineValue", dp, fValue, getCurrentTime());
  DebugTN("onlineValue() - done: ", dp);

}


/**
  @brief Ruleset to guess the datatype of the unit being trended.

  @todo Find a more robust way to do this, by saving some property in the DPE,
        which also distinguishes between mbar for vacuum and other instruments
        and different voltage levels

  @see unVacTrending_ScaleType

  @param sDpName The name of the dpe for which the scale type has to be guessed
  @return An element of the enumerate UNVACTRENDING_SCALETYPE
*/
unVacTrending_ScaleType unVacTrending_guessScaleType(string sDpeName,  unVacTrending_ScaleType defaultScale = unVacTrending_ScaleType::UNITLESS /* UNVACTRENDING_SCALETYPE.UNITLESS */){
  string sUnit = dpGetUnit(sDpeName);
  string sDpType = dpTypeName(sDpeName);

  if(sUnit == UNVACTRENDING_UNIT_MBAR){
    // If the unit is defined as mbar, always mbar scale
    return unVacTrending_ScaleType::MBARVAC;
  }
  else if(sUnit == "%"){
	// If the unit is defined as percentage, always mbar scale
	return unVacTrending_ScaleType::PERCENTAGE;
  }
  else if(sUnit == UNVACTRENDING_UNIT_VOLT || sUnit == UNVACTRENDING_UNIT_VOLT_ALTERNATIVE){
	// If the unit is defined as volts, depending on the device selected the scaling
    if(sDpType == UN_CONFIG_CPC_VP_TP_DPT_NAME){
      return unVacTrending_ScaleType::VOLTVERYLOW;
    }
	else if(sDpType == UN_CONFIG_CPC_VP_I_DPT_NAME){
      return unVacTrending_ScaleType::VOLTHIGH;
    }

    return unVacTrending_ScaleType::VOLTLOW;
  }
  else if(sUnit == UNVACTRENDING_UNIT_AMPERE || sUnit == UNVACTRENDING_UNIT_AMPERE_ALTERNATIVE){
	// If the unit is defined as amperes, depending on the device selected the scaling
    if(sDpType == UN_CONFIG_CPC_VP_TP_DPT_NAME){
      return unVacTrending_ScaleType::AMPSMEDIUM;
    }
	else if(sDpType == UN_CONFIG_CPC_VP_I_DPT_NAME){
      return unVacTrending_ScaleType::AMPSLOW;
    }

    // Default to medium amps
    return unVacTrending_ScaleType::AMPSMEDIUM;
  }
  else if(dpElementType(sDpeName) == DPEL_UINT){
    // Show default status object if integer type of
    return unVacTrending_ScaleType::STATUSOBJECT;
  }
  else if(sDpType == UN_CONFIG_CPC_ONOFF_DPT_NAME){
	return unVacTrending_ScaleType::BIT;
  }

  return defaultScale;
}

/**
@brief Get maximum time range in second according to the timestamp of the oldest value archived
@param[in] dp The name of DPE for which max time range shall be found
@return
*/
int unVacTrending_getDpArchiveMaxTimeRange(string dp)
{
  //DebugTN("Start search first time for " + dp);
  time now = getCurrentTime() - 100;
  time firstTime = now;
  time startTime = now;
  int delta = 200 * 24 * 60 * 60;
  while(true)
  {
    dyn_time times;
    dyn_float values;
    dynClear(times);
    dpGetPeriod(startTime, startTime + 1, 1, dp, values, times);
    //DebugTN("Query for ", startTime, dynlen(times), delta, times);
    if(dynlen(times) == 0)
    {
      delta /= 2;
      if(delta < 3600)
      {
        break;
      }
      startTime = firstTime - delta;
    }
    else
    {
      firstTime = times[1];
      if(firstTime > startTime)
      {
        break; // No more times before start time
      }
      startTime -= delta;
    }
  }
  //DebugTN("First time for " + dp, firstTime);
  if(firstTime < now)
  {
    firstTime <= 7200; // Search was stopped at this precision
  }
  int maxTimeRangeSecond = now - firstTime;
  //DebugTN("maxTimeRangeSecond " + dp, maxTimeRangeSecond);
  return maxTimeRangeSecond;

  /*
  ** Method dpGet(dpe + ":_start.._online", firstTime) works well for file archving,
  ** but useless for RDB archiving.
  ** Method dpGetPeriod(farInThePast) works somtimes, not always, the logic is not clear to me.
  */
}
