
#uses "vacuumExtension/classes/DeviceListCriteria.ctl"

/**@file
 *
 * This package contains functions to support device list functionality for UNICOS devices
 *
 * @author Leonid Kopylov (TE-VSC)
 * @date   March 2017
 */

/***************************************************************************************************
************************ Control of state history content (list of devices shown) ******************
***************************************************************************************************/

/** Return name of DP type to store device list criteria

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL
*/
string vacDeviceList_getConfigDpType() {
  return "_VacDeviceListConfig";
}

/** Create new DP for storing device list config, return name of new DP

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL
*/
string vacDeviceList_createConfigDp(dyn_string &exceptionInfo) {
  string sDpTypeName = vacDeviceList_getConfigDpType();
  dyn_string dsExistingDps = dpNames("*", sDpTypeName);
  string sDpName;
  for(int n = dynlen(dsExistingDps) + 1 ; n < 10000000 ; n++) {
    sprintf(sDpName, "%s_%d", sDpTypeName, n);
    if(!dpExists(sDpName)) {
      break;
    }
  }
  dpCreate(sDpName, sDpTypeName);
  dyn_errClass err = getLastError();
  if(dynlen(err) > 0)
  {
    dyn_string exceptionInfo;
    fwException_raise(exceptionInfo, 
      "ERROR", "vacDeviceList_createConfigDp(): failed to create DP <" + sDpName + ":" + err , "");
    return "";
  }
  return sDpName;
}

bool vacDeviceList_deviceMatchesNature(string sAliasName, dyn_string dsSelectedNatures) {
  string dpName = unGenericDpFunctions_dpAliasToName(sAliasName);
//DebugTN("after  unGenericDpFunctions_dpAliasToName",   dpName, sAliasName); 
  string domain, nature;
  unGenericDpFunctions_getGroups(dpName, domain, nature);
//DebugN("after  unGenericDpFunctions_getGroups",   dpName, domain, nature); 
  if(dynContains(dsSelectedNatures, nature) <= 0) {
    return false;
  }
  return true;
}

/** Check if device matches given criteria.
  Note: this function does not preform real check, it finds and call device type
    specific function if one is defined. If function is not defined - device does not
    match criteria

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDeviceName  input, name of device to check, can be DP or alias name
@param criteria input, the criteria to check
*/
bool vacDeviceList_deviceMatchesCriteria(string sDeviceName, shared_ptr<DeviceListCriteria> criteria,
                                         string &sStateString, string &sStateColor, string &sValueString,
                                         dyn_string &dsStateDpes, dyn_anytype &daStateValues) {

  string sDpName;
  string sTypeName = vacDeviceList_getTypeName(sDeviceName, sDpName);
  if(sDpName == "") {
    return false;
  }
  sStateString = "";
  sStateColor = "";
  sValueString = "";
  dynClear(dsStateDpes);
  dynClear(daStateValues);

  // Unfrotunately, the following piece of code is only found inside unSimpleAnimation_WidgetConnect(),
  // there is no public function to do this. So, code is copied to this place 
  string sFunctionName = sTypeName + "_WidgetInitStatics";
  if(isFunctionDefined(sFunctionName)) {
    execScript("main(string deviceName) {" + sFunctionName + "(deviceName); }", makeDynString(), sDpName);
  }

  sFunctionName = sTypeName + "_MatchesCriteria";
  if(!isFunctionDefined(sFunctionName)) {
    return false;
  }
  
  // It is assumed that WidgetDPEs return enough DPEs to know everything about current device state
  string sFrontEnd = unGenericObject_GetPLCNameFromDpName(sDpName);
  if(sFrontEnd != "") {
    string sSystemName = unGenericDpFunctions_getSystemName(sDpName);
    sFrontEnd = sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sFrontEnd;
  }

  dsStateDpes = unSimpleAnimation_WidgetDPEs(sDpName, sFrontEnd);
  dpGet(dsStateDpes, daStateValues);
  dyn_errClass deErr = getLastError();
  if(dynlen(deErr) > 0) {
    for(int idx = 1 ; idx < dynlen(deErr) ; idx++) {
      throw(deErr[idx]);
    }
    return false;
  }
  
  // Check if device matches criteria - type-specific function will do the real job
  bool bResult = callFunction(sFunctionName, sDpName, dsStateDpes, daStateValues, criteria, sStateString, sStateColor, sValueString);

  if(bResult) {
    // Save DPE names and values, used for decision. Not all of DPEs are needed, but only
    // those which are responsible for state.
    dyn_string dsStateOnlyDpes;
    vacuumExtension_StateDPEs(sDpName, dsStateOnlyDpes, false);
    for(int iDpeIdx = dynlen(dsStateDpes) ; iDpeIdx > 0 ; iDpeIdx--) {
      if(dynContains(dsStateOnlyDpes, dsStateDpes[iDpeIdx]) <= 0) {
        dynRemove(dsStateDpes, iDpeIdx);
        dynRemove(daStateValues, iDpeIdx);
      }
    }
  }
 
  return bResult;
}

/////////////////////////////////////////////////////////////////////////////////////////
////////////// Implementation of type-specific functions ////////////////////////////////
////////////// Finally shall be moved to corresponding <type>.ctl libraries /////////////
/////////////////////////////////////////////////////////////////////////////////////////


bool vacDeviceList_checkPressureGrowCriteria(string sDpeName, shared_ptr<DeviceListCriteria> criteria) {

  time tStartTime, tEndTime;
  float fGrowFactor;
  int iSpikeInterval;
  if(!criteria.getPressureGrowParams(tStartTime, tEndTime, fGrowFactor, iSpikeInterval)) {
    return false;
  }

  float fMinValue = 1E+10;  // Something very large
  time tFirstLargeValueTime;  // Time when value first exceeded threshold, originally - empty

  while(tStartTime < tEndTime) {

    time tQueryEndTime = tStartTime + 4 * 60 * 60;  // Query in 4 hours interval
    dyn_float dfValues;
    dyn_time dtTimes;
    dpGetPeriod(tStartTime, tQueryEndTime, 1, sDpeName, dfValues, dtTimes);
    dyn_errClass deErr = getLastError();
    if(dynlen(deErr) > 0) {  // dpGetPeriod() has failed - the same as access error
      criteria.macthAccessErr();
      return criteria.isResult();
    }
    
    if(dynlen(dfValues) > 0) {  // Got some history
      if(dtTimes[dynlen(dtTimes)] > tStartTime) {  // Can happen that only time before interval exists
        if(vacDeviceList_checkValueGrow(dfValues, dtTimes, fGrowFactor, iSpikeInterval, fMinValue, tFirstLargeValueTime)) {
          return true;
        }
      }
    }
 
    tStartTime = tQueryEndTime;
  }
  return false;
}

bool vacDeviceList_checkValueGrow(dyn_float dfValues, dyn_time dtTimes, float fGrowFactor,
                                          int iSpikeInterval, float &fMinValue, time &tFirstLargeValueTime) {

  int iValueCount = dynlen(dfValues);
  time tEmptyTime;

  for(int idx = 1 ; idx <= iValueCount ; idx++) {

    float fValue = dfValues[idx];
    if((fValue <= 0) || (fValue >= 10000)) {
      continue;
    }

    if(fValue < fMinValue) {
      fMinValue = fValue;
    }
    else if((fValue / fMinValue) > fGrowFactor) {
      if(iSpikeInterval <= 0) {
        return true;
      }
      if(tFirstLargeValueTime == tEmptyTime) {
        tFirstLargeValueTime = dtTimes[idx];
      }
      else {
        int iDeltaTime = dtTimes[idx] - tFirstLargeValueTime;
        if(iDeltaTime > iSpikeInterval) {
          return true;
        }
      }
    }
    else {
      tFirstLargeValueTime = tEmptyTime;
    }

  }
  return false;
}

private string vacDeviceList_getTypeName(string sDeviceName, string &sDpName) {

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "vacDeviceList",
                                "vacDeviceList_getTypeName(" + sDeviceName + "): start");
  if(!dpExists(sDeviceName)) {
    sDpName = unGenericDpFunctions_dpAliasToName(sDeviceName);
    unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "vacDeviceList",
                                  "vacDeviceList_getTypeName(" + sDeviceName + "): DP from alias", sDpName);
    if(!dpExists(sDpName)) {
      unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "vacDeviceList",
                                   "vacDeviceList_getTypeName(" + sDeviceName + "): no DP found");
      return "";
    }
    sDpName = dpSubStr(sDpName, DPSUB_DP); 
  }
  else {
    sDpName = sDeviceName;
  }
  return dpTypeName(sDpName);
}




string vacDeviceList_GetDpType(string sDeviceName, string &sDpName)
{
  
  string sTypeName = vacDeviceList_getTypeName(sDeviceName, sDpName);
  return(sTypeName);
}
/***************************************************************************************************
************************ Integral state (and color) for device *************************************
************************ Different functions for different types ***********************************
***************************************************************************************************/


/**
Calculate string ('integral device state') to be shown in the table
plus color string, corresponding to this state

@param sDeviceName (in) device name (DP name or alias)
@param sStateString (out) device state string
@param sColorString (out) device color string

*/
void vacDeviceList_getStateAndColor(string sDeviceName, string &sStateString, string &sColorString) {
  
  sStateString = "";
  sColorString = "";
  dyn_string   stateDpeList;
  dyn_uint     dpeValues;
  string sDpName;
  string sTypeName = vacDeviceList_getTypeName(sDeviceName, sDpName);
  if(sDeviceName == "") {
    return;
  }
  vacuumExtension_StateDPEs(sDpName, stateDpeList, false);
  int dpeNum = dynlen(stateDpeList);
  for(int n = 1; n <= dpeNum; n++ )
  {
    uint value;
    dpGet(stateDpeList[n], value);
    dynAppend(dpeValues, value); 
  }
  vacuumExtension_StateMainColorAndString(sDpName, stateDpeList, dpeValues, sColorString, sStateString);
}

