/**@file
 *
 * This package contains functions to support state history for UNICOS devices
 *
 * @author Leonid Kopylov (TE-VSC)
 * @date   March 2017
 */

/***************************************************************************************************
************************ Control of state history content (list of devices shown) ******************
***************************************************************************************************/

/** Find name of DP for control of state history content for current UI;
  create DP if it does not exist yet.

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param bClearContent  input, true if content of DP shall be cleared
@param sDpName output, resulting DP name
@param exceptionInfo output, error description will be returned here in case of problem
*/
void vacStateHistory_getCtlDpName(bool bClearContent, string &sDpName, dyn_string &exceptionInfo) {

  string sDpTypeName = "_VacStateHistory";

  sDpName = sDpTypeName + "_" + myManNum();

  // Create DP if needed
  if(!dpExists(sDpName)) {
    if(dpCreate(sDpName, sDpTypeName) < 0) {
      fwException_raise(exceptionInfo, 
        "ERROR",
        "vacStateHistory_getCtlDpName(): dpCreate(" + sDpName + "," + sDpTypeName + ") returned error",
        "");
      return;
    }
    dyn_errClass err = getLastError();
    if(dynlen(err) > 0) {
      fwException_raise(exceptionInfo, 
        "ERROR",
        "vacStateHistory_getCtlDpName(): Failed to create DP <" + sDpName + ">: " + err,
        "");
      return;
    }
    // Check if DP was really created
    if(!dpExists(sDpName)) {
      fwException_raise(exceptionInfo, 
        "ERROR",
        "vacStateHistory_getCtlDpName(): DP <" + sDpName + "> was not created",
        "");
      return;
    }
  }

  if(bClearContent) {
    dpSet(sDpName + ".DpList", makeDynString());
    dyn_errClass err = getLastError();
    if(dynlen(err) > 0) {
      fwException_raise(exceptionInfo, 
        "ERROR",
        "vacStateHistory_getCtlDpName(): Failed to clear content of <" + sDpName + ">: " + err,
        "");
      return;
    }
  }
}

/** Add given DP(s) to list of DPs shown in state history

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dsDpList  input, List of DP names to be added to state history
@param bHasToOpenPanel input, true if state history panel has to opened
@param exceptionInfo output, error description will be returned here in case of problem
*/
void vacStateHistory_addDevice(dyn_string dsDpList, bool bHasToOpenPanel, dyn_string &exceptionInfo) {

  int iListLen = dynlen(dsDpList);
  if(iListLen == 0) {
    return;
  }

  string sModuleName = "State_History";
  bool isOpen = false;
  string sCtlDpName;

  if(bHasToOpenPanel) {
    isOpen = isModuleOpen(sModuleName);
    // clean dpList & DLL data pool according to state of dialog
    if(!isOpen) {
      vacStateHistory_getCtlDpName(true, sCtlDpName, exceptionInfo);
      if(dynlen(exceptionInfo) > 0) {
        return;
      }
    }
  }
  
  if(sCtlDpName == "") {
    vacStateHistory_getCtlDpName(false, sCtlDpName, exceptionInfo);
    if(dynlen(exceptionInfo) > 0) {
      return;
    }
  }

  // Get current content of history
  dyn_string dsCurrentDpList;
  dpGet(sCtlDpName + ".DpList", dsCurrentDpList);
  dyn_errClass err = getLastError();
  if(dynlen(err) > 0) {
    fwException_raise(exceptionInfo, 
      "ERROR", "VacStateHistoryAddDp(): failed to read current config from < " + sCtlDpName +
      " >:" + err, "");
    return;
  }

  dynAppend(dsCurrentDpList, dsDpList);
  dynUnique(dsCurrentDpList);
	
  // Set new values for DP
  dpSet(sCtlDpName + ".DpList", dsCurrentDpList);
  err = getLastError();
  if(dynlen(err) > 0) {
    fwException_raise(exceptionInfo, 
      "ERROR", "vacStateHistory_addDdevice(): failed to write new config to < " + sCtlDpName + " >:" + err, "");
    return;
  }
	
  if((!isOpen) && bHasToOpenPanel) {
    dyn_string dollars = makeDynString();
    ModuleOnWithPanel(sModuleName, -1, -1, 0, 0, 0, 0, "None", "vision/vacPanels/stateHistory/StateHistory.pnl", "State_History", dollars);
  }
  if(bHasToOpenPanel) {
    stayOnTop(true, sModuleName);
    delay(0, 100);
    stayOnTop(false, sModuleName);
  }
}

/** Remove given DP from list of DPs shown in state history panel.

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpName  input, the name of DP to be removed
@param exceptionInfo output, error description will be returned here in case of problem
*/
// 
void vacStateHistory_removeDevice(string sDeviceName, dyn_string &exceptionInfo) {

  string  sCtlDpName;
	
  vacStateHistory_getCtlDpName(false, sCtlDpName, exceptionInfo);
  if(dynlen(exceptionInfo) > 0) {
    return;
  }

  // Get current content of state history
  dyn_string dsCurrentDpList;
  dpGet(sCtlDpName + ".DpList", dsCurrentDpList);
  dyn_errClass err = getLastError();
  if(dynlen(err) > 0) {
    fwException_raise(exceptionInfo, 
      "ERROR", "vacStateHistory_removeDevice(): failed to read current config from < " + sCtlDpName + " >:" + err, "");
    return;
  }

  // Find DP in list
  int iDpIndex = dynContains(dsCurrentDpList, sDeviceName);
  if(iDpIndex < 1) {
    return;		// Not found
  }

  // Correct list, write back to DP
  dynRemove(dsCurrentDpList, iDpIndex);
  dpSet(sCtlDpName + ".DpList", dsCurrentDpList);
  err = getLastError();
  if(dynlen(err) > 0) {
    fwException_raise(exceptionInfo, 
      "ERROR", "vacStateHistory_removeDevice(): failed to write new config to < " + sCtlDpName + " >:" + err, "");
  }
}
